/*
 * main.h - global definitions
 */
#ifndef MAIN_H_
#define MAIN_H_

/* paths and prefixes */
#ifndef PREFIX_DIR
 #define PREFIX_DIR		"/usr/local"
#endif
#define DAEMON_DIR		"lincond"
#ifndef LOCALSTATE_DIR
 #define LOCALSTATE_DIR		PREFIX_DIR "/var/run"
#endif
#ifndef CONFIG_DIR
 #define CONFIG_DIR		PREFIX_DIR "/etc"
#endif

/* paths to directories and files */
#define CONFIG_DEFAULT		"lincond.conf"
#define SOCKFILE_DEFAULT	"lincond.sock"

/* Lincon limits */
#define LINCON_SLOT_MAX		7
#define LINCON_SLOT_MIN		1
#define REMOTE_SLOT_MAX		9
#define REMOTE_SLOT_MIN		1

#define DCON_ADDRESS_MIN	0
#define DCON_ADDRESS_MAX	255

#define TIMEOUT_MS_MIN		10	/* 10 mseconds */
#define TIMEOUT_MS_MAX		10000	/* 10 seconds */
#define TIMEOUT_MS_DEFAULT	100	/* reasonable default */

/* type of module's bus */
typedef enum {
	MODULE_BUS_SERIAL,
	MODULE_BUS_PARALLEL,
} module_bus_t;

typedef enum {
	/* parallel bus modules */
	MODULE_DI4,
	MODULE_DI8,
	MODULE_DI16,
	MODULE_DI32,
	MODULE_DO4,
	MODULE_DO8,
	MODULE_DO16,
	MODULE_DO32,
	MODULE_DIO8,
	MODULE_DIO16,
	/* serial bus modules  */
	MODULE_DI4_SERIAL,
	MODULE_DI8_SERIAL,
	MODULE_DI16_SERIAL,
	MODULE_DI32_SERIAL,
	MODULE_DO4_SERIAL,
	MODULE_DO8_SERIAL,
	MODULE_DO16_SERIAL,
	MODULE_DO32_SERIAL,
	MODULE_DIO8_SERIAL,
	MODULE_DIO16_SERIAL,
	MODULE_AI4_SERIAL,
	MODULE_AI8_SERIAL,
	MODULE_AO4_SERIAL,
	MODULE_AO8_SERIAL,
} module_type_t;

typedef enum {
	MODULE_UNIT_RAW,
	MODULE_UNIT_EU,
	MODULE_UNIT_FSR,
	MODULE_UNIT_OHM,
} module_unit_t;

struct module_options {
	char 			*name_str;		/* module known name */
	module_type_t		module_type;		/* type identifier */
	module_bus_t		module_bus;		/* bus type */
	unsigned int		occupied_inputs;	/* how many bytes in input image this module occupies */
	unsigned int		occupied_outputs;	/* how many bytes in output image this module occupies */
	unsigned int		nr_input_channels;	/* number of input channels this module has */
	unsigned int		nr_output_channels;	/* number of output channels this module has */
};

typedef enum {
	STATION_DISABLED,
	STATION_DISCONNECTED,
	STATION_READY,
	STATION_BUSY,
	STATION_DONE,
} station_state_t;

typedef enum {
	MODULE_DISABLED,
	MODULE_OK,
} module_state_t;

typedef enum {
	MODULE_REQUEST_NONE,
	MODULE_REQUEST_AO_READ,
	MODULE_REQUEST_AO_WRITE,
} module_request_state_t;

struct station {
	char			port[PATH_MAX];
	char			speed[NAME_MAX];
	int			serial_fd;
	int			response_tout;
	struct timeout		*tout;
	station_state_t		state;
	struct slist		*modules_list;
	struct slist		*current_module;
	struct buffer		*rb;
	struct buffer		*wb;
	struct io_context	*io;
	int			has_parallel_bus :1;
	int			use_csum :1;
};

#define MAX_AO_CHANS		12

struct module {
	int			slot;
	int			address;
	module_type_t		type;
	module_state_t		state;
	size_t			inputs_offset;
	size_t			outputs_offset;
	module_unit_t		unit;
	int			output_changed :1;	/* option to output only changed values to AO channels */

	/* here module keeps his state data */
	module_request_state_t	request;		/* requested data state */
	unsigned int		last_ao_chan;		/* last read/write serial AO channel */
	unsigned int		readback_ao_chan;	/* bits indicate AO channel to readback */
	float			output[MAX_AO_CHANS];	/* previous AO channels output values */
};

#endif
